// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>
#import "NNTag.h"
#import "NNTags.h"

extern NSString * const NNTaggableObjectUpdate;

/**
abstract class representing a taggable object (normally a file
 but can be anything really)
 
 important: take care to save the files to backup storage on dealloc in all subclasses!
 
 tags must be loaded by subclass!
 */
@interface NNTaggableObject : NSObject <NSCopying> {
	
	NSMutableSet			*tags;
	NNTags					*globalTags;
	
	int						retryCount;
	
	NSString				*displayName;
	NSString				*contentType;
	NSString				*contentTypeIdentifier;
	NSArray					*contentTypeTree;
	NSDate					*lastUsedDate;

	NSNotificationCenter	*nc;
		
	NSWindow				*errorWindow;
	
	BOOL					manageFilesAutomatically;
	BOOL					manageFiles;
		
}

- (NSMutableSet*)tags;

/** 
should be used internally only (for example to
implement copying), as it does not save the new
tags to the backup storage

if you want to do that, please call initiateSave 
afterwards
*/
- (void)setTags:(NSMutableSet*)someTags;

- (int)retryCount;
- (void)incrementRetryCount;
- (void)setRetryCount:(int)i;

- (NSString *)displayName;
- (void)setDisplayName:(NSString *)aDisplayName;
- (NSString *)contentType;
- (void)setContentType:(NSString *)aContentType;
- (NSString *)contentTypeIdentifier;
- (void)setContentTypeIdentifier:(NSString *)aContentTypeIdentifier;
- (NSArray *)contentTypeTree;
- (void)setContentTypeTree:(NSArray *)aContentTypeTree;
- (NSDate *)lastUsedDate;
- (void)setLastUsedDate:(NSDate *)aDate;

- (void)addTag:(NNTag*)tag;
- (void)addTags:(NSArray*)someTags;
- (void)removeTag:(NNTag*)tag;
- (void)removeTags:(NSArray*)someTags;
- (void)removeAllTags;

/**
call this if you want to save to harddisk,
 don't call saveTags directly
 */
- (void)initiateSave;

/**
will be called when files are scheduled for file managing,
 abstract method does nothing, subclass may implement on demand
  - only called if pref is set
 */
- (void)handleFileManagement;

/**
must be implemented by subclass,
 save tags to backing storage
 @returns success or failure
 */
- (BOOL)saveTags;

/**
will be called on renaming
 must be implemented in subclass
 @param newName new name for taggable object
 @param window error window
 */
- (void)renameTo:(NSString*)newName errorWindow:(NSWindow*)window;

/**
will be called on moving to trash
 must be implemented in subclass
 @param flag YES trashes, NO does nothing
 @param window error window
 */
- (void)moveToTrash:(BOOL)flag errorWindow:(NSWindow *)window;

/**
checks if new name for object is valid
 for example:
 NNFile needs to check if the filename is free in the directory
 
 must be implemented by subclass!
 @param newName proposed name
 @returns YES if valid, else NO
 */
- (BOOL)validateNewName:(NSString*)newName;

- (BOOL)shouldManageFiles;
- (void)setShouldManageFiles:(BOOL)flag;
- (BOOL)shouldManageFilesAutomatically;
- (void)setShouldManageFilesAutomatically:(BOOL)flag;

+ (id)replaceMetadataValue:(id)attrValue forAttribute:(NSString *)attrName;

@end