// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>

/**
treat this class as the <b>abstract</b> superclass for all Tags,
 most of the methods are not implemented here, 
 subclasses need to overwrite!
 */
@interface NNTag : NSObject <NSCoding, NSCopying>
{
	NSString *name;
	NSString *query;
	NSCalendarDate *lastClicked;
	NSCalendarDate *lastUsed;
	unsigned long clickCount;
	unsigned long useCount;
}

// these functions need tobe implemented by subclass

/** 
<b>must overwrite</b>

@param other NSObject to compare to
@returns YES if equal; NO otherwise
*/
- (BOOL)isEqual:(id)other;

/** 
<b>must overwrite</b>

@param zone NSZone to use
@returns copied NNTag subclass
*/
- (id)copyWithZone:(NSZone *)zone;

/** 
<b>must overwrite</b>

tag rating should be between 0.0 and infinity

@returns tag rating
*/
- (float)absoluteRating;

/** 
<b>must overwrite</b>

@param otherTag tag to compare rating to
@returns a value in [0.0,1.0]
*/
- (float)relativeRatingToTag:(NNTag*)otherTag;

/** 
<b>must overwrite</b>

use this method if you want to implement a tag renaming which updates the backing storage

@param aName new tag name
*/
- (void)renameTo:(NSString*)aName;

/** 
<b>must overwrite</b>

this method is called when the tag should be removed from the backing storage
*/
- (void)remove;


// these functions have been implemented, but
// subclasses may overwrite

/** 
<b>may overwrite</b>

initializes the tag with the given name
@param aName name for the new tag
@returns NNTag subclass with aName
*/
- (id)initWithName:(NSString*)aName;

/** 
<b>may overwrite</b>

returns the tag name	
*/
- (NSString*)name;

/** 
<b>may overwrite</b>
	
@returns the query in NSPredicate syntax
*/
- (NSString*)query;

/** 
<b>may overwrite</b>

@returns the query in the MDQuery syntax
*/
- (NSString*)queryInSpotlightSyntax;

/**
executes a spotlight query for the tag. blocks the thread until results have been gathered.
 @returns NNTaggableObjects array for tag
 */
- (NSArray*)taggedObjects;


/** 
<b>may overwrite</b>

@returns the time when this tag was most recently clicked
*/
- (NSCalendarDate*)lastClicked;

/** 
<b>may overwrite</b>

@param clickDate new last clicked date
*/
- (void)setLastClicked:(NSCalendarDate*)clickDate;

/** 
<b>may overwrite</b>

@returns the time when this tag was most recently assigned to a NNTaggableObject
*/
- (NSCalendarDate*)lastUsed;

/** 
<b>may overwrite</b>

@param useDate new last used date
*/
- (void)setLastUsed:(NSCalendarDate*)useDate;

/** 
<b>may overwrite</b>

@returns how often this tag was clicked
*/
- (unsigned long)clickCount;

/** 
<b>may overwrite</b>

@returns how often this tag was used
*/
- (unsigned long)useCount;

/** 
<b>may overwrite</b>

@param aName new tag name
*/
- (void)setName:(NSString*)aName;

/** 
<b>may overwrite</b>

@param aQuery query in spotlight syntax
*/
- (void)setQuery:(NSString*)aQuery;

/** 
<b>may overwrite</b>

increments the click count
*/
- (void)incrementClickCount;

/** 
<b>may overwrite</b>

increments the use count
*/
- (void)incrementUseCount;

/** 
<b>may overwrite</b>

decrements the use count
*/
- (void)decrementUseCount;

/** 
<b>may overwrite</b>

@param count new use count
*/
- (void)setUseCount:(int)count;

/** 
<b>may overwrite</b>

@param count new click count
*/
- (void)setClickCount:(int)count;

@end
