// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>
#import "NNTag.h"
#import "NNTaggableObject.h"
#import "NNFile.h"

#import "NNQueryBundle.h"
#import "NNQueryFilter.h"
#import "NNSelectedTags.h"
#import "NSFileManager+TrashFile.h"


/** Posted when the receiver begins with the initial result-gathering phase of the query. */
extern NSString * const NNQueryDidStartGatheringNotification;

/** Posted when new results have been found */
extern NSString * const NNQueryGatheringProgressNotification;

/** Posted when the receiver’s results have changed during the live-update phase of the query. */
extern NSString * const NNQueryDidUpdateNotification;

/** Posted when the receiver has finished with the initial result-gathering phase of the query. */
extern NSString * const NNQueryDidFinishGatheringNotification;

/** Posted when the query is reset - needed when results are emptied */
extern NSString * const NNQueryDidResetNotification;

/** Posted when the receiver's grouping attributes have changed. */
//extern NSString * const NNQueryGroupingAttributesDidChange;


/**
NNQuery is mainly a wrapper around the NSMetadataQuery class, and shares a lot of its API. 
 There are additional methods for manipulating the search, as NNQuery uses NNTag instances instead of 
 directly setting the NSPredicate.
 */
@interface NNQuery : NSObject
{
	id						delegate;
	NSMetadataQuery			*mdquery;
	
	NSDictionary			*simpleGrouping;
	
	NNSelectedTags			*tags;
	
	NSPredicate				*predicate;
	
	NSMutableDictionary		*bundles;
	NSArray					*bundlingAttributes;
	
	NSArray					*sortDescriptors;			// Currently not used
	
	NSMutableDictionary		*filterDict;
	NSMutableArray			*filters;
	
	NSMutableArray			*plainResults;
	NSMutableArray			*flatPlainResults;
	NSMutableArray			*filteredResults;
	NSMutableArray			*flatFilteredResults;
	
	NSWindow				*errorWindow;
}

/**
@returns a new query - use setTags: to set tags for searching
 */
- (id)init;

/**
designated initializer - call this one if you want to start searching for tags
 immediately
 @param otherTags tags to search for
 @returns NNQuery ready to search for otherTags
 */
- (id)initWithTags:(NNSelectedTags*)otherTags;

/**
@returns tags the query is currently prepared to search for
 */
- (NNSelectedTags*)tags;

/**
@param otherTags tags to search for
 */
- (void)setTags:(NNSelectedTags*)otherTags;

//wrapper methods
/**
start the query
 @returns YES when succesful, NO otherwise
 */
- (BOOL)startQuery;

/**
stop the query
 */
- (void)stopQuery;

/**
temporarily stops the query from gathering results - 
 use this if you e.g. want to display intermediate results
 */
- (void)disableUpdates;

/**
reenables result gathering - use this after disableUpdates
 */
- (void)enableUpdates;

/**
@returns YES if query is started, NO otherwise
 */
- (BOOL)isStarted;

/**
@returns YES if the query is stopped, NO otherwise
 */
- (BOOL)isStopped;

/**
this returns YES if the query is started and updating was not
 disabled
@returns YES if query is gathering results, NO otherwise
 */
- (BOOL)isGathering;

/**
@returns number of results currently found
 */
- (unsigned)resultCount;

/**
@param idx index of the result to return
 @returns result at position idx
 */
- (id)resultAtIndex:(unsigned)idx;

/**
@returns results that may contain a hierarchical tree of bundles
 or a flat list of items. This depends on the current bundling
 attributes.
 */
- (NSArray *)results;

/**
@returns results as flat array
 */
- (NSArray *)flatResults;

/**
@returns bundling attributes as array
*/
- (NSArray *)bundlingAttributes;

/**
sets new bundling attributes
@param attributes the new attributes to use
*/
- (void)setBundlingAttributes:(NSArray *)attributes;

/**
@return array of NNQueryFilters
 */
- (NSArray*)filters;

/**
@param someFilters filters to use when executing the query
 */
- (void)setFilters:(NSMutableArray*)someFilters;

/**
adds a filter to the query
 @param filter filter to add
 */
- (void)addFilter:(NNQueryFilter*)filter;

/**
adds multiple filters to the query
 @param someFilters filters to add
 */
- (void)addFilters:(NSArray*)someFilters;

/**
removes a filter from the query
 @param filter filter to remove
 */
- (void)removeFilter:(NNQueryFilter*)filter;

/**
removes all active query filters
 */
- (void)removeAllFilters;

/**
applies new filter. may be set at any time even if the query is currently active.
 @param flag if YES filter is applied
 @param filterValues array of values to use for the filter, e.g. "DOCUMENTS" or "PDF"
 @param attribute DEPRECATED - currently supposed to be "kMDItemContentTypeTree"
 @param newAttributes DEPRECATED
 */
-   (void)filterResults:(BOOL)flag
			usingValues:(NSArray *)filterValues
   forBundlingAttribute:(NSString *)attribute
  newBundlingAttributes:(NSArray *)newAttributes;

/**
returns YES if there are results for the given filter
 @param filterValues array of values to use for the filter, e.g. "DOCUMENTS" or "PDF"
 @param attribute attribute to use, e.g. "kMDItemContentTypeTree"
 @returns YES if there are results, NO otherwise
 */
- (BOOL)hasResultsUsingFilterWithValues:(NSArray *)filterValues
                   forBundlingAttribute:(NSArray *)attribute;

/**
@returns YES if there's currently a filter
 */
- (BOOL)hasFilter;


@end